import PIL.Image
import PIL.TiffTags

def dumpexif(im, fix = True):
    exif = im.getexif()

    if fix:
        ImageWidthExifTag = 0x0100
        if (ImageWidthExifTag not in exif):
            exif[ImageWidthExifTag] = im.width

        ImageLengthExifTag = 0x0101
        if (ImageLengthExifTag not in exif):
            exif[ImageLengthExifTag] = im.height

    for tag, value in exif.items():
        if isinstance(value, bytes):
            value = value.decode('utf-8')

        tag_info = PIL.TiffTags.lookup(tag)
        print(f'{tag_info.name}: {value}')



#!/usr/bin/env python3

import argparse

import pydingo as pdg

parser = argparse.ArgumentParser()
parser.add_argument('host', nargs = '?')
parser.add_argument('--port', type = int, default = 8080)

args = parser.parse_args()

with pdg.Client(args.host, args.port) as cli:
    vid = cli.video()
    for vid_frame in vid.decode(video = 0):
        print(vid_frame)



#!/usr/bin/env python3

import argparse

import pydingo as pdg

parser = argparse.ArgumentParser()
parser.add_argument('host', nargs = '?')
parser.add_argument('--port', type = int)
parser.add_argument('--frequency', type = float, default = 1)
parser.add_argument('--expiry', type = float, default = 15)
parser.add_argument('--show', type = int)

args = parser.parse_args()

ims = []
with pdg.Client(args.host, args.port) as cli:
    @cli.image_every(args.frequency, args.expiry)
    def callback(im):
        if (len(ims) > 0):
            print()

        pdg.dumpexif(im)
        ims.append(im)

    if (args.show is not None):
        im = ims[args.show]
        im.show()



from setuptools import setup

#from setuptools_scm import get_version
# 'write_to': 'version.py'
 
setup(name = 'sts-pydingo', description = 'Client library', author = 'See-Through Scientific',
    packages = ['pydingo'], install_requires = ['pillow', 'tornado', 'zeroconf'],
    use_scm_version = {'local_scheme': 'no-local-version'})



import importlib
import queue

try:
    import av
except ImportError:
    pass

from .client_connection import ClientConnection

class QueueIO(object):
    def __init__(self):
        self.finished = False
        self.queue = queue.Queue()

    def read(self, n):
#        print('read(%d)' % n)
        if self.finished:
            return b''

        data = self.queue.get()
        if not data:
            self.finished = True
        assert len(data) <= n
        return data

    def write(self, buf):
        self.queue.put(buf)

class VideoSession(object):
    def __init__(self):
        self.qio = QueueIO()

    def add(self, buf):
        self.qio.write(buf)

def av_video(hostname, port):
    conn = ClientConnection()
    conn.vid_session = VideoSession()

    conn.connect(hostname, port)
    conn.remote_call('video', conn.new_id())

    return av.open(conn.vid_session.qio), conn



#!/usr/bin/env python3

import argparse

import pydingo as pdg

parser = argparse.ArgumentParser()
parser.add_argument('host', nargs = '?')
parser.add_argument('--port', type = int)
parser.add_argument('--policy')

args = parser.parse_args()

with pdg.Client(args.host, args.port) as cli:
    if (args.policy is not None):
        cli.set_image_background_policy(args.policy)

    im_bg = cli.image_background()
    im_bg.show()



import asyncio

class RPCError(Exception):
    def __init__(self, message, code):
        Exception.__init__(self, message)

        self.code = code

class RPCSession(object):
    def __init__(self):
        self.next_id = 0
        self.futs = {}

    def get_id(self, callback = None):
        value = self.next_id
        self.next_id += 1

        loop = asyncio.get_event_loop()

        fut = loop.create_future()
        if (callback is not None):
            fut.add_done_callback(lambda fut: callback(fut.result()))

        self.futs[value] = fut

        return value

    def set_result(self, id, value):
        fut = self.futs.pop(id)
        fut.set_result(value)

    def set_error(self, id, value):
        raise RPCError(**value)

    def set(self, dtl, bufs):
        if ('error' in dtl):
            self.set_error(dtl['error'])
        else:
            self.set_result(dtl['id'], (dtl['result'], bufs))



#!/usr/bin/env python3

import argparse

import pydingo as pdg

parser = argparse.ArgumentParser()
parser.add_argument('host', nargs = '?')
parser.add_argument('--port', type = int)

args = parser.parse_args()

with pdg.Client(args.host, args.port) as cli:
    im = cli.image()
    pdg.dumpexif(im)

    im.show()
